@@ -3,7 +3,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2014-2023 Univention GmbH
+# Copyright 2014-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -32,23 +32,23 @@
 
 """Generate `country_data.json`"""
 
-from __future__ import print_function
+from __future__ import annotations
 
 import json
 from argparse import ArgumentParser, FileType
-from typing import Any, Dict  # noqa: F401
+from typing import Any, Dict
 
 import _util
 
 
-if __name__ == '__main__':
+def main() -> None:
     parser = ArgumentParser(description=__doc__)
     parser.add_argument("outfile", type=FileType("w"))
     parser.add_argument("locales", nargs="+")
     opt = parser.parse_args()
 
     print('generating country data...')
-    country_data = {}  # type: Dict[str, Dict[str, Any]]
+    country_data: Dict[str, Dict[str, Any]] = {}
 
     country_default_lang = _util.get_country_default_language()
     for icountry, ilang in country_default_lang.items():
@@ -76,3 +76,7 @@ if __name__ == '__main__':
     opt.outfile.write("\n")
 
     print('... done :)')
+
+
+if __name__ == '__main__':
+    main()
