@@ -7,7 +7,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2011-2023 Univention GmbH
+# Copyright 2011-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -34,6 +34,8 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 import copy
 import json
 import locale as _locale
@@ -44,10 +46,9 @@ import subprocess
 import threading
 import time
 import traceback
+from typing import Any, Dict, List
 
 import lxml.etree
-import notifier
-import notifier.threads
 import psutil
 
 import univention.config_registry
@@ -55,7 +56,7 @@ from univention.lib.admember import connectionFailed, failedADConnect, lookup_ad
 from univention.lib.i18n import Locale, Translation
 from univention.management.console.log import MODULE
 from univention.management.console.modules import Base, UMC_Error
-from univention.management.console.modules.decorators import sanitize, simple_response
+from univention.management.console.modules.decorators import SimpleThread, sanitize, simple_response, threaded
 from univention.management.console.modules.mixins import ProgressMixin
 from univention.management.console.modules.sanitizers import IntegerSanitizer, PatternSanitizer, StringSanitizer
 
@@ -110,19 +111,18 @@ class Instance(Base, ProgressMixin):
         # reset umask to default
         os.umask(0o022)
 
-    def init(self):
+    def init(self) -> None:
         os.putenv('LANG', str(self.locale))
         _locale.setlocale(_locale.LC_ALL, str(self.locale))
         if not util.is_system_joined():
             self._preload_city_data()
 
-    def _preload_city_data(self):
+    def _preload_city_data(self) -> None:
         util.get_city_data()
         util.get_country_data()
 
-    def _get_localized_label(self, label_dict):
-        # return the correctly loca
-        return label_dict.get(self.locale.language) or label_dict.get('en', '') or label_dict.get('', '')
+    def _get_localized_label(self, label_dict: Dict[str, str]) -> str:
+        return label_dict.get(self.locale.language, '') or label_dict.get('en', '') or label_dict.get('', '')
 
     def ping(self, request):
         if request.options.get('keep_alive'):
@@ -131,14 +131,14 @@ class Instance(Base, ProgressMixin):
         self.finished(request.id, None)
 
     @simple_response
-    def close_browser(self):
+    def close_browser(self) -> bool:
         try:
             with open('/var/cache/univention-system-setup/browser.pid', 'rb') as fd:
                 pid = int(fd.readline().strip())
                 process = psutil.Process(pid)
                 process.kill()
                 return True
-        except IOError as exc:
+        except OSError as exc:
             MODULE.warn('cannot open browser PID file: %s' % (exc,))
         except ValueError as exc:
             MODULE.error('browser PID is not a number: %s' % (exc,))
@@ -155,7 +155,7 @@ class Instance(Base, ProgressMixin):
         return util.load_values(self.locale.language)
 
     @simple_response
-    def save_keymap(self, layout=None):
+    def save_keymap(self, layout=None) -> bool:
         """
         Set the systems x-keymap according to
         request.options[keymap]
@@ -220,7 +220,7 @@ class Instance(Base, ProgressMixin):
                 restart = False
                 if not run_hooks:
                     MODULE.info('Check whether ip addresses have been changed')
-                    for ikey, _ival in values.items():
+                    for ikey in values.keys():
                         if RE_IPV4.match(ikey) or RE_IPV6_DEFAULT.match(ikey) or RE_SSL.match(ikey):
                             restart = True
                             break
@@ -241,7 +241,7 @@ class Instance(Base, ProgressMixin):
             finally:
                 obj._finishedLock.release()
 
-        def _finished(thread, result):
+        def _finished(thread, result) -> None:
             if self.__keep_alive_request:
                 self.finished(self.__keep_alive_request.id, None)
                 self.__keep_alive_request = None
@@ -253,12 +253,12 @@ class Instance(Base, ProgressMixin):
                 self._progressParser.current.critical = True
                 self._finishedResult = True
 
-        thread = notifier.threads.Simple('save', notifier.Callback(_thread, request, self), _finished)
-        thread.run()
+        thread = SimpleThread('save', _thread, _finished)
+        thread.run(request, self)
         self.finished(request.id, None)
 
     @simple_response
-    def join(self, values=None, dcname=None, username=None, password=None):
+    def join(self, values=None, dcname: str | None = None, username: str | None = None, password: str | None = None) -> None:
         '''
         Join and reconfigure the system according to the values specified in the dict given as
         option named "values".
@@ -274,7 +274,7 @@ class Instance(Base, ProgressMixin):
         # create a status file that indicates that save has been triggered
         util.create_status_file()
 
-        def _thread(obj, username, password):
+        def _thread(obj, username: str, password: str) -> bool:
             # acquire the lock until the scripts have been executed
             self._finishedResult = False
             obj._finishedLock.acquire()
@@ -306,7 +306,7 @@ class Instance(Base, ProgressMixin):
             finally:
                 obj._finishedLock.release()
 
-        def _finished(thread, result):
+        def _finished(thread, result) -> None:
             if self.__keep_alive_request:
                 self.finished(self.__keep_alive_request.id, None)
                 self.__keep_alive_request = None
@@ -318,53 +318,49 @@ class Instance(Base, ProgressMixin):
                 self._progressParser.current.critical = True
                 self._finishedResult = True
 
-        thread = notifier.threads.Simple('join', notifier.Callback(_thread, self, username, password), _finished)
-        thread.run()
-        return
+        thread = SimpleThread('join', _thread, _finished)
+        thread.run(self, username, password)
 
-    def check_finished(self, request):
+    @threaded
+    def check_finished(self, request) -> None:
         """
         Check whether the join/setup scripts are finished. This method implements a long
         polling request, i.e., the request is only finished at the moment when all scripts
         have been executed or due to a timeout. If it returns because of the timeout, a new
         try can be started.
         """
-        def _thread(request, obj):
-            def progress_info(state, **kwargs):
-                info = {
-                    'component': state.fractionName,
-                    'info': state.message,
-                    'errors': state.errors,
-                    'critical': state.critical,
-                    'steps': state.percentage,
-                }
-                info.update(kwargs)
-                MODULE.info('Progress state: %(steps).1f%% - %(component)s - %(info)s' % info)
-                return info
-            # acquire the lock in order to wait for the join/setup scripts to finish
-            # do this for 30 sec and then return anyway
-            SLEEP_TIME = 0.200
-            WAIT_TIME = 30
-            ntries = WAIT_TIME / SLEEP_TIME
-            while not obj._finishedLock.acquire(False):
-                if ntries <= 0 or self._progressParser.changed and self._progressParser.current:
-                    state = self._progressParser.current
-                    return progress_info(state, finished=False)
-                time.sleep(SLEEP_TIME)
-                ntries -= 1
+        def progress_info(state, **kwargs):
+            info = {
+                'component': state.fractionName,
+                'info': state.message,
+                'errors': state.errors,
+                'critical': state.critical,
+                'steps': state.percentage,
+            }
+            info.update(kwargs)
+            MODULE.info('Progress state: %(steps).1f%% - %(component)s - %(info)s' % info)
+            return info
+        # acquire the lock in order to wait for the join/setup scripts to finish
+        # do this for 30 sec and then return anyway
+        SLEEP_TIME = 0.200
+        WAIT_TIME = 30
+        ntries = WAIT_TIME / SLEEP_TIME
+        while not self._finishedLock.acquire(False):
+            if ntries <= 0 or self._progressParser.changed and self._progressParser.current:
+                state = self._progressParser.current
+                return progress_info(state, finished=False)
+            time.sleep(SLEEP_TIME)
+            ntries -= 1
 
-            obj._finishedLock.release()
+        self._finishedLock.release()
 
-            # scripts are done, return final result
-            # return all errors that we gathered throughout the setup
-            state = self._progressParser.current
-            return progress_info(state, finished=obj._finishedResult)
-
-        thread = notifier.threads.Simple('check_finished', notifier.Callback(_thread, request, self), notifier.Callback(self.thread_finished_callback, request))
-        thread.run()
+        # scripts are done, return final result
+        # return all errors that we gathered throughout the setup
+        state = self._progressParser.current
+        return progress_info(state, finished=self._finishedResult)
 
     @simple_response(with_flavor=True)
-    def validate(self, values=None, flavor=None):
+    def validate(self, values: Dict | None = None, flavor: str | None = None):
         '''
         Validate the specified values given in the dict as option named "values".
         Return a dict (with variable names as key) of dicts with the structure:
@@ -389,7 +385,8 @@ class Instance(Base, ProgressMixin):
         # helper functions
         # TODO: 'valid' is not correctly evaluated in frontend
         # i.e. if valid you may continue without getting message
-        def _check(key, check, message, critical=True):
+        def _check(key: str, check, message: str, critical: bool = True) -> None:
+            assert values is not None
             if key not in values:
                 return
             if not check(values[key]):
@@ -399,7 +396,7 @@ class Instance(Base, ProgressMixin):
                     'key': key,
                 })
 
-        def _append(key, message):
+        def _append(key: str, message: str) -> None:
             MODULE.warn('Validation failed for key %s: %s' % (key, message))
             messages.append({
                 'key': key,
@@ -559,7 +556,7 @@ class Instance(Base, ProgressMixin):
         """Return a list of all available time zones."""
         try:
             fd = open('/usr/share/univention-system-setup/locale/timezone')
-        except EnvironmentError:
+        except OSError:
             MODULE.error('Cannot find locale data for timezones in /usr/share/univention-system-setup/locale')
             self.finished(request.id, None)
             return
@@ -571,7 +568,7 @@ class Instance(Base, ProgressMixin):
     @simple_response
     def lang_keyboard_model(self):
         """Return a list of all available keyboard models."""
-        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))
+        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))  # noqa: S320
         models = tree.xpath("//model")
 
         model_result = [{
@@ -584,7 +581,7 @@ class Instance(Base, ProgressMixin):
     @simple_response
     def lang_keyboard_layout(self):
         """Return a list of all available keyboard layouts."""
-        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))
+        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))  # noqa: S320
         layouts = tree.xpath("//layout")
 
         layout_result = [{
@@ -601,7 +598,7 @@ class Instance(Base, ProgressMixin):
     def lang_keyboard_variante(self, keyboardlayout):
         """Return a list of all available keyboard variantes."""
         variante_result = []
-        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))
+        tree = lxml.etree.parse(open('/usr/share/X11/xkb/rules/base.xml'))  # noqa: S320
         layouts = tree.xpath("//layout")
 
         for layout in layouts:
@@ -657,8 +654,8 @@ class Instance(Base, ProgressMixin):
             self._net_apply_running -= 1
             self.finished(request.id, True)
 
-        thread = notifier.threads.Simple('net_apply', notifier.Callback(_thread, self), _finished)
-        thread.run()
+        thread = SimpleThread('net_apply', _thread, _finished)
+        thread.run(self)
 
     @simple_response
     def net_apply_check_finished(self):
@@ -704,7 +701,7 @@ class Instance(Base, ProgressMixin):
 
     @sanitize(pattern=StringSanitizer(), max_results=IntegerSanitizer(minimum=1, default=5))
     @simple_response
-    def find_city(self, pattern, max_results):
+    def find_city(self, pattern: str, max_results: int) -> List | None:
         pattern = pattern.lower()
         MODULE.info('pattern: %s' % pattern)
         if not pattern:
@@ -763,11 +760,11 @@ class Instance(Base, ProgressMixin):
         return matches
 
     @simple_response
-    def apps_query(self):
+    def apps_query(self) -> List[Dict[str, Any]]:
         return util.get_apps(True)
 
     @simple_response
-    def check_domain(self, role, nameserver):
+    def check_domain(self, role: str, nameserver: str) -> Dict[str, Any]:
         result = {}
         if role == 'ad':
             try:
@@ -796,7 +793,7 @@ class Instance(Base, ProgressMixin):
         return result
 
     @simple_response
-    def check_domain_join_information(self, domain_check_role, role, dns, nameserver, address, username, password):
+    def check_domain_join_information(self, domain_check_role: str, role: str, dns: str, nameserver: str, address: str, username: str, password: str) -> Dict[str, str]:
         result = {}
         if domain_check_role == 'ad':
             domain = util.check_credentials_ad(nameserver, address, username, password)
@@ -816,13 +813,13 @@ class Instance(Base, ProgressMixin):
         return result
 
     @simple_response
-    def check_school_information(self, hostname, address, username, password):
+    def check_school_information(self, hostname: str, address: str, username: str, password: str):
         return check_for_school_domain(hostname, address, username, password)
 
     @simple_response
-    def check_repository_accessibility(self):
+    def check_repository_accessibility(self) -> List[str]:
         return get_unreachable_repository_servers()
 
     @simple_response
-    def check_uid(self, uid, role, address, username, password):
+    def check_uid(self, uid: str, role: str, address: str, username: str, password: str) -> bool:
         return check_if_uid_is_available(uid, role, address, username, password)
