@@ -7,7 +7,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2018-2023 Univention GmbH
+# Copyright 2018-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -34,9 +34,8 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-from __future__ import absolute_import
 
-from pipes import quote
+from shlex import quote
 from subprocess import PIPE, Popen
 
 from ldap.filter import filter_format
@@ -45,12 +44,11 @@ from univention.management.console.log import MODULE
 from univention.management.console.modules.setup.util import _temporary_password_file
 
 
-def check_if_uid_is_available(uid, role, address, username, password):
+def check_if_uid_is_available(uid: str, role: str, address: str, username: str, password: str) -> bool:
     """
     check if either the UID it not yet taken at all
     or it is already taken (by our previous self) and still matches the server role
     """
-    # type: (str, str, str, str, str) -> bool
     filter_s = filter_format("(&(objectClass=person)(uid=%s)(!(univentionServerRole=%s)))", [uid, role])
     rcmd = 'univention-ldapsearch -LLL %s 1.1' % (quote(filter_s),)
     with _temporary_password_file(password) as password_file:
@@ -64,5 +62,5 @@ def check_if_uid_is_available(uid, role, address, username, password):
         process = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
         stdout, stderr = process.communicate()
         if process.wait() or stderr:
-            MODULE.error("Failed checking uid=%s role=%s: %s" % (uid, role, stderr))
+            MODULE.error("Failed checking uid=%s role=%s: %r" % (uid, role, stderr))
     return not stdout.strip()
