@@ -1,3 +1,10 @@
+#
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
+# SPDX-FileCopyrightText: 2024 Univention GmbH
+# SPDX-License-Identifier: AGPL-3.0-only
+
 from univention.management.console.modules.setup.netconf import Phase
 
 
@@ -6,7 +13,7 @@ class PhaseFlushOldAddresses(Phase):
 
     priority = 55
 
-    def pre(self):
+    def pre(self) -> None:
         super(PhaseFlushOldAddresses, self).pre()
         for _name, iface in self.changeset.old_interfaces.all_interfaces:
             self.call(["ip", "addr", "flush", iface.name])
