@@ -1,3 +1,10 @@
+#
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
+# SPDX-FileCopyrightText: 2024 Univention GmbH
+# SPDX-License-Identifier: AGPL-3.0-only
+
 from ldap import LDAPError
 
 from univention.admin import modules
@@ -10,14 +17,14 @@ class PhaseLdapDhcp(LdapChange):
 
     priority = 48
 
-    def post(self):
+    def post(self) -> None:
         try:
             self.open_ldap()
             self._create_subnet()
         except (LDAPError, UniventionBaseException) as ex:
             self.logger.warning("Failed LDAP: %s", ex)
 
-    def _create_subnet(self):
+    def _create_subnet(self) -> None:
         ipv4 = self.changeset.new_interfaces.get_default_ipv4_address()
         if not ipv4:
             return
