@@ -6,7 +6,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2011-2023 Univention GmbH
+# Copyright 2011-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -40,7 +40,7 @@ dcname=""
 runcleanup=true
 
 while [ "$#" -gt 0 ]; do
-	case $1 in
+	case "$1" in
 		--dcname)
 			dcname="$2"
 			shift 2
@@ -75,6 +75,7 @@ done
 SETUP_LOG="/var/log/univention/setup.log"
 JOIN_LOG="/var/log/univention/join.log"
 
+# shellcheck source=setup_utils.sh
 . /usr/lib/univention-system-setup/scripts/setup_utils.sh
 
 echo "Reset UCR variables"
@@ -91,21 +92,19 @@ run-parts -v /usr/lib/univention-system-setup/scripts/25_defaultlocale
 
 # Do not allow the UMC to be restarted, webserver is OK, but
 # make sure the webserver uses the same certificates during setup.
-# The variables are removed in during cleanup
-/usr/share/univention-updater/disable-apache2-umc --exclude-apache
+# The variables are removed during cleanup
+/usr/share/univention-updater/disable-apache2-umc
 # Do not change apache certificate when installing via debian installer
 eval "$(univention-config-registry shell)"
-if [ "$system_setup_boot_installer" != "true" ]; then
-	certificate="$(mktemp -p /var/cache/univention-system-setup)"
-	key="$(mktemp -p /var/cache/univention-system-setup)"
-	ca="$(mktemp -p /var/cache/univention-system-setup)"
-	cp "/etc/univention/ssl/$(ucr get hostname).$(ucr get domainname)/cert.pem" "$certificate"
-	cp "/etc/univention/ssl/$(ucr get hostname).$(ucr get domainname)/private.key" "$key"
-	cp "/etc/univention/ssl/ucsCA/CAcert.pem" "$ca"
+if [ "${system_setup_boot_installer:-}" != "true" ]; then
+	ssl="$(mktemp -d -p /var/cache/univention-system-setup)"
+	install -m 0644 "/etc/univention/ssl/$hostname.$domainname/cert.pem" "$ssl/cert.pem"
+	install -m 0600 "/etc/univention/ssl/$hostname.$domainname/private.key" "$ssl/private.key"
+	install -m 0644 "/etc/univention/ssl/ucsCA/CAcert.pem" "$ssl/CAcert.pem"
 	ucr set \
-		apache2/ssl/certificate="$certificate" \
-		apache2/ssl/key="$key" \
-		apache2/ssl/ca="$ca"
+		apache2/ssl/certificate="$ssl/cert.pem" \
+		apache2/ssl/key="$ssl/private.key" \
+		apache2/ssl/ca="$ssl/CAcert.pem"
 fi
 
 # Re-create the system uuid
@@ -114,9 +113,9 @@ fi
 # Re-create sources.list files before installing the role packages
 #  https://forge.univention.org/bugzilla/show_bug.cgi?id=28089
 ucr commit /etc/apt/sources.list.d/*
-apt-get update
+apt-get -q update
 
-# Bug #45896: Make the UCS 4.3 DVD trusted
+# Bug #45896: Make the UCS ISO trusted
 sed -i 's|deb cdrom:\[UCS |deb \[trusted=yes\] cdrom:\[UCS |' /etc/apt/sources.list
 
 # Install the server package
@@ -132,22 +131,24 @@ hostname=$(get_profile_var "hostname")
 # set domainame
 domainname=$(get_profile_var "domainname")
 [ -n "$domainname" ] && univention-config-registry set domainname="$domainname"
-# set ldap/basee
+# set ldap/base
 ldap_base=$(get_profile_var "ldap/base")
 [ -n "$ldap_base" ] && univention-config-registry set ldap/base="$ldap_base"
 # set windows domain
 windows_domain=$(get_profile_var "windows/domain")
 [ -n "$windows_domain" ] && univention-config-registry set windows/domain="$windows_domain"
 
+# restart udm to be sure the new settings apply
+. /usr/share/univention-lib/base.sh
+stop_udm_cli_server
+
 eval "$(univention-config-registry shell)"
 
 # The ldap server join script must create the Administrator account
 if [ "$server_role" = "domaincontroller_master" ]; then
 	p=$(get_profile_var "root_password")
 	if [ -n "$p" ]; then
-		if [ ! -e /var/lib/univention-ldap ]; then
-			mkdir -p /var/lib/univention-ldap
-		fi
+		install -m 0755 -d /var/lib/univention-ldap
 		echo -n "$p" >/var/lib/univention-ldap/root.secret
 		chmod 600 /var/lib/univention-ldap/root.secret
 	fi
@@ -159,13 +160,13 @@ fi
 /usr/lib/univention-system-setup/scripts/10_basis/18root_password
 
 if [ "$server_role" = "domaincontroller_master" ]; then
-	realm="$(echo $domainname | tr "[:lower:]" "[:upper:]")"
-	univention-config-registry set ldap/server/name="$hostname.$domainname" \
-						ldap/master="$hostname.$domainname" \
-						kerberos/adminserver="$hostname.$domainname" \
-						kerberos/realm="$realm" \
-						mail/alias/root="systemmail@$hostname.$domainname"
-
+	realm="$(echo "$domainname" | tr "[:lower:]" "[:upper:]")"
+	univention-config-registry set \
+		ldap/server/name="$hostname.$domainname" \
+		ldap/master="$hostname.$domainname" \
+		kerberos/adminserver="$hostname.$domainname" \
+		kerberos/realm="$realm" \
+		mail/alias/root="systemmail@$hostname.$domainname"
 fi
 
 info_header "create-ssh-keys" "$(gettext "Recreating SSH keys")"
@@ -207,9 +208,9 @@ fi
 echo "Starting re-configuration of network"
 run-parts -v -a --network-only -a --appliance-mode -- /usr/lib/univention-system-setup/scripts/30_net
 eval "$(ucr shell proxy/http proxy/https proxy/no_proxy)"
-[ -n "$proxy_http" ] && export http_proxy="$proxy_http"
-[ -n "$proxy_https" ] && export https_proxy="$proxy_https"
-[ -n "$proxy_no_proxy" ] && export no_proxy="$proxy_no_proxy"
+[ -n "${proxy_http:-}" ] && export http_proxy="$proxy_http"
+[ -n "${proxy_https:-}" ] && export https_proxy="$proxy_https"
+[ -n "${proxy_no_proxy:-}" ] && export no_proxy="$proxy_no_proxy"
 
 run-parts -v /usr/lib/univention-system-setup/scripts/35_timezone
 
@@ -230,12 +231,14 @@ else
 	# Create them in any case, as apache2 will not start otherwise
 	# These should be treated as dummy certificates,
 	# as a new certificate will be created and downloaded when joining a UCS domain
+
+	# shellcheck source=/dev/null
 	. /usr/share/univention-ssl/make-certificates.sh
 	cd "$SSLBASE"
 	echo "Creating base ssl certificate"
 	fqdn="$hostname.$domainname"
-	gencert "$SSLBASE/$fqdn" "$fqdn" "$days"
-	if getent group "DC Backup Hosts" 2>&1 >/dev/null
+	gencert "$SSLBASE/$fqdn" "$fqdn" "${days:-365}"
+	if getent group "DC Backup Hosts" >/dev/null 2>&1
 	then
 		chgrp -R "DC Backup Hosts" "$SSLBASE/$name"
 		chmod -R g+rX "$SSLBASE/$name"
@@ -272,53 +275,44 @@ if [ $? -ne 1 ]; then
 	fi
 	(
 		if [ "$server_role" = "domaincontroller_master" ]; then
-			mkdir -p /var/univention-join/ /usr/share/univention-join/
+			install -m 0755 -d /var/univention-join /usr/share/univention-join
 			rm -f /var/univention-join/joined /var/univention-join/status
 			touch /var/univention-join/joined /var/univention-join/status
 			ln -sf /var/univention-join/joined /usr/share/univention-join/.joined
 			ln -sf /var/univention-join/status /usr/lib/univention-install/.index.txt
 
 			for i in /usr/lib/univention-install/*.inst; do
-				echo "Configure $(basename "$i") $(LC_ALL=C date)" | tee -a "$JOIN_LOG"
-				# test
-				$i 2>&1 | tee -a "$JOIN_LOG"
-			done
+				echo "Configure $(basename "$i") $(LC_ALL=C date)"
+				"$i" 2>&1
+			done | tee -a "$JOIN_LOG"
 		else
-			if [ -n "$dcaccount" -a -n "$password_file" ]; then
+			if [ -n "$dcaccount" ] && [ -n "$password_file" ]; then
 				# Copy to a temporary password file, because univention-join
 				# will copy the file to the same directory on the Primary Directory Node
 				# with the given user credentials. This will not work.
 				pwd_file="$(mktemp)"
-				cp "$password_file" "$pwd_file"
-				if [ -n "$dcname" ]; then
-					/usr/share/univention-join/univention-join -dcname "$dcname" -dcaccount "$dcaccount" -dcpwd "$pwd_file"
-				else
-					/usr/share/univention-join/univention-join -dcaccount "$dcaccount" -dcpwd "$pwd_file"
-				fi
+				install -m 0600 "$password_file" "$pwd_file"
+				/usr/share/univention-join/univention-join ${dcname:+-dcname "$dcname"} -dcaccount "$dcaccount" -dcpwd "$pwd_file"
 				rm -f "$pwd_file"
 			fi
 		fi
 	) 2>&1 | (
-		# parse the output for lines "^Configure .*" which indicate that a join
-		# script is being executed
-		while read line; do
-			if [ "${line#Configure }" != "$line" ]; then
-				# found line starting with "Configure " ... parse the join script name
+		while read -r line
+		do
+			case "$line" in
+			'Configure '*)  # parse the join script name
 				joinScript=${line#Configure }
 				joinScript=${joinScript%%.inst*}
-				progress_msg "$(gettext "Configure") $(basename $joinScript)"
+				progress_msg "$(gettext "Configure") $(basename "$joinScript")"
 				progress_next_step
-			fi
-			if [ "${line#__JOINERR__}" != "$line" ]; then
-				# found line indicating join failed. output
-				echo "$line"
-				continue
-			fi
-			if [ "${line#* Message:  }" != "$line" ]; then
-				# found line indicating join failed. output
+				;;
+			__JOINERR__*)  # join failed output
+				;;
+			*' Message:  '*)  # join failed output
 				progress_join_error "${line#* Message:  }"
 				continue
-			fi
+				;;
+			esac
 			echo "$line"
 		done
 	)
